#include <stdio.h>
#include <string.h>
#include <math.h>
// libgd's header file
#include <gd.h>

// test main function
int main(int argc, char **argv)
{
	gdImage *bm;
	int x, y;
	int i_x, i_y, f_x, f_y;
	int black, dgreen, lgreen, white;
	float min, max;
	float f0, f1;
	int j;
	int fn = -1;
	FILE *fp;

	if (argc != 2)
	{
		fprintf(stderr,
			"graph: wrong argc\n");
		return 1;
	}

	if (!strcmp(argv[1], "load")) fn = 0;
	else if (!strcmp(argv[1], "mem")) fn = 1;
	else if (!strcmp(argv[1], "swap")) fn = 2;

	if (fn == -1)
	{
		fprintf(stderr,
			"graph: function must be one of load, mem, swap\n");
		return 2;
	}

	// create image
	bm = gdImageCreate(600, 200);

	if (!bm)
	{
		fprintf(stderr, "graph: gdImageCreate: couldn't allocate image\n");
		return 3;
	}

	// create colours
	black = gdImageColorAllocate(bm, 0, 0, 0);
	dgreen = gdImageColorAllocate(bm, 0, 128, 0);
	lgreen = gdImageColorAllocate(bm, 0, 192, 0);
	white = gdImageColorAllocate(bm, 255, 255, 255);

	// draw axes/grid
	for (x = 0; x < 600; x += 10)
		gdImageLine(bm, x, 0, x, 200, (x % 50) ? dgreen : lgreen);

	for (y = 0; y < 200; y += 25)
		gdImageLine(bm, 0, y, 600, y, (y % 50) ? dgreen : lgreen);

	// now draw a graph
	switch(fn)
	{
	case 0:
		fp = fopen("/var/status/load.history", "rt");
		min = 0.0f; max = 4.0f;
		break;

	case 1:
		fp = fopen("/var/status/mem.history", "rt");
		min = 0.0f; max = 1.0f;
		break;

	case 2:
		fp = fopen("/var/status/swap.history", "rt");
		min = 0.0f; max = 1.0f;
		break;

	default:
		return 3;	// can't happen ;)
	}

	fscanf(fp, "%f", &f0);

	// the history files contain 61 minutes of data
	for (j = 1; j < 61; j++)
	{
		fscanf(fp, "%f", &f1);

		i_x = (j - 1) * 10;
		f_x = j * 10;
		i_y = 199 - (int)((200.0f * (f0 - min)) / (max - min));
		f_y = 199 - (int)((200.0f * (f1 - min)) / (max - min));

		if (i_y == -1) i_y = 0;
		if (f_y == -1) f_y = 0;

		if (i_y > 199) i_y = 199;
		if (f_y > 199) f_y = 199;

		gdImageLine(bm, i_x, i_y, f_x, f_y, white);

		f0 = f1;
	}

	fclose(fp);

	// dump output
	gdImageGif(bm, stdout);

	gdImageDestroy(bm);
}

